@@ -1,6 +1,9 @@
 """AsusRouter constants module"""
 
-from asusrouter.dataclass import Key
+from enum import Enum
+from typing import Any
+
+from asusrouter.dataclass import Key, SearchKey
 from asusrouter.util.converters import (
     bool_from_any,
     bool_or_int,
@@ -13,6 +16,14 @@ from asusrouter.util.converters import (
     time_from_delta,
 )
 
+### ENUMS
+class Merge(str, Enum):
+    """Merge enumerator for getting data"""
+
+    ALL = "all"
+    ANY = "any"
+
+
 ### API VALUES
 AR_USER_AGENT = "asusrouter--DUTUtil-"
 AR_API = [
@@ -22,21 +33,334 @@ AR_API = [
 ]
 
 ### VALUES
+ACTION_MODE = "action_mode"
 AIMESH = "aimesh"
+APPLY = "apply"
+BOOTTIME = "boottime"
+BRIDGE = "bridge"
 CLIENTS = "clients"
 CONNECTION_TYPE = "connection_type"
+CPU = "cpu"
+CPU_USAGE = "cpu_usage"
+DATA = "data"
+DEVICEMAP = "devicemap"
+DIAG = "diag"
+DHCP = "dhcp"
+ENDPOINTS = "endpoints"
+ERRNO = "errno"
+FIRMWARE = "firmware"
 GUEST = "guest"
+GWLAN = "gwlan"
+HOOK = "hook"
+INFO = "info"
 IP = "ip"
+ISO = "iso"
+LACP = "lacp"
+LAN = "lan"
+LINK_RATE = "link_rate"
+LOAD_AVG = "load_avg"
 MAC = "mac"
+MAIN = "main"
+MEMORY_USAGE = "memory_usage"
+MONITOR = "monitor"
+NAME = "name"
+NETDEV = "netdev"
+NETWORK = "network"
+NETWORKMAPD = "networkmapd"
 NODE = "node"
+NPMCLIENT = "npmclient"
+NVRAM = "nvram"
+NVRAM_GET = "nvram_get"
 ONBOARDING = "onboarding"
+ONLINE = "online"
+PORT = "port"
+PORTS = "ports"
+QTN = "qtn"
+RAM = "ram"
 RSSI = "rssi"
+RX = "rx"
+SIM = "sim"
+STATE = "state"
+STATUS = "status"
+SYS = "sys"
+SYSINFO = "sysinfo"
+TEMPERATURE = "temperature"
+TIMESTAMP = "timestamp"
+TOTAL = "total"
+TX = "tx"
+UNKNOWN = "unknown"
+UPDATE_CLIENTS = "update_clients"
+USAGE = "usage"
+USED = "used"
+USB = "usb"
+VPN = "vpn"
+VPN_CLIENT = "vpn_client"
+VPN_SERVER = "vpn_server"
+WAN = "wan"
+WANLINK_STATE = "wanlink_state"
+WIRED = "wired"
+WLAN = "wlan"
+WLAN_2GHZ = "2ghz"
+WLAN_5GHZ = "5ghz"
+WLAN_5GHZ2 = "5ghz2"
+WLAN_6GHZ = "6ghz"
+
+### ASUS DATA TYPES
+ETHERNET_PORTS = "ethernet_ports"
+PORT_STATUS = "port_status"
+
+### ENDPOINTS
+ENDPOINT = {
+    DEVICEMAP: "ajax_status.xml",
+    ETHERNET_PORTS: "ajax_ethernet_ports.asp",
+    FIRMWARE: "detect_firmware.asp",
+    HOOK: "appGet.cgi",
+    # NETWORKMAPD: "update_networkmapd.asp",
+    ONBOARDING: "ajax_onboarding.asp",
+    PORT_STATUS: "get_port_status.cgi",
+    SYSINFO: "ajax_sysinfo.asp",
+    TEMPERATURE: "ajax_coretmp.asp",
+    UPDATE_CLIENTS: "update_clients.asp",
+    VPN: "ajax_vpn_status.asp",
+}
+
+ENDHOOKS: dict[str, Any] = {
+    MAIN: {
+        "cpu_usage": "appobj",
+        "memory_usage": "appobj",
+        "netdev": "appobj",
+        "wanlink_state": "appobj",
+    },
+    NVRAM: None,
+}
+
+ENDPOINT_ARGS = {
+    PORT_STATUS: {
+        MAC: f"{NODE}_{MAC}",
+    },
+}
+
+### History-dependent values in the monitor to be removed to prevent errors
+HD_DATA: tuple[tuple[str, ...], ...] = (
+    (MAIN, CPU),
+    (MAIN, NETWORK),
+)
+
+### REQUIREMENTS
+# Don't create loops
+CONST_REQUIRE_MONITOR = {
+    WLAN: MAIN,
+}
+MONITOR_REQUIRE_CONST = {
+    NVRAM: WLAN,
+}
+
+### ASUS NUMERICS, RANGES & MAPS
+
+# Map of the parameters for connected device
+# value_to_find: [where_to_search, method to convert]
+# List is sorted by importance with the most important first
+MAP_CONNECTED_DEVICE: dict[str, list[SearchKey]] = {
+    "connected_since": [
+        SearchKey("wlConnectTime", time_from_delta),
+    ],
+    CONNECTION_TYPE: [
+        SearchKey(CONNECTION_TYPE),
+        SearchKey("isWL", int_from_str),
+    ],
+    GUEST: [
+        SearchKey(GUEST),
+        SearchKey("isGN", int_from_str),
+    ],
+    "internet_mode": [
+        SearchKey("internetMode"),
+    ],
+    "internet_state": [
+        SearchKey("internetState", bool_from_any),
+    ],
+    IP: [
+        SearchKey(IP),
+    ],
+    "ip_method": [
+        SearchKey("ipMethod"),
+    ],
+    MAC: [
+        SearchKey(MAC),
+    ],
+    NAME: [
+        SearchKey("nickName"),
+        SearchKey(NAME),
+        SearchKey(MAC),
+    ],
+    NODE: [
+        SearchKey(NODE),
+    ],
+    ONLINE: [
+        SearchKey(ONLINE),
+        SearchKey("isOnline", bool_from_any),
+    ],
+    RSSI: [
+        SearchKey(RSSI, int_from_str),
+    ],
+    "rx_speed": [
+        SearchKey("curRx", float_from_str),
+    ],
+    "tx_speed": [
+        SearchKey("curTx", float_from_str),
+    ],
+}
+MAP_CPU: tuple[Key, ...] = (Key(TOTAL), Key(USAGE, USED))
+MAP_NETWORK: tuple[Key, ...] = (
+    Key("INTERNET", WAN),
+    Key("INTERNET1", USB),
+    Key("WIRED", WIRED),
+    Key("BRIDGE", BRIDGE),
+    Key("WIRELESS0", WLAN_2GHZ),
+    Key("WIRELESS1", WLAN_5GHZ),
+    Key("WIRELESS2", WLAN_5GHZ2),
+    Key("WIRELESS3", WLAN_6GHZ),
+    Key("LACP1", f"{LACP}1"),
+    Key("LACP2", f"{LACP}2"),
+)
+MAP_NVRAM = {
+    WLAN: [
+        SearchKey("wl{}_auth_mode_x"),
+        SearchKey("wl{}_bw"),
+        SearchKey("wl{}_channel"),
+        SearchKey("wl{}_chanspec"),
+        SearchKey("wl{}_closed", bool_from_any),
+        SearchKey("wl{}_country_code"),
+        SearchKey("wl{}_crypto"),
+        SearchKey("wl{}_gmode_check"),
+        SearchKey("wl{}_maclist_x"),
+        SearchKey("wl{}_macmode"),
+        SearchKey("wl{}_mbo_enable"),
+        SearchKey("wl{}_mfp"),
+        SearchKey("wl{}_nmode_x"),
+        SearchKey("wl{}_optimizexbox_ckb"),
+        SearchKey("wl{}_radio", bool_from_any),
+        SearchKey("wl{}_radius_ipaddr"),
+        SearchKey("wl{}_radius_key"),
+        SearchKey("wl{}_radius_port"),
+        SearchKey("wl{}_ssid"),
+        SearchKey("wl{}_wpa_gtk_rekey"),
+        SearchKey("wl{}_wpa_psk"),
+    ],
+    GWLAN: [
+        SearchKey("wl{}_akm"),
+        SearchKey("wl{}_ap_isolate"),
+        SearchKey("wl{}_auth"),
+        SearchKey("wl{}_auth_mode"),
+        SearchKey("wl{}_auth_mode_x"),
+        SearchKey("wl{}_bridge"),
+        SearchKey("wl{}_bss_enabled", bool_from_any),
+        SearchKey("wl{}_bss_maxassoc", int_from_str),
+        SearchKey("wl{}_bw_dl", int_from_str),  # Bandwidth limit download
+        SearchKey("wl{}_bw_enabled", bool_from_any),  # Bandwidth limit switch
+        SearchKey("wl{}_bw_ul", int_from_str),  # Bandwidth limit upload
+        SearchKey("wl{}_closed", bool_from_any),
+        SearchKey("wl{}_crypto"),
+        SearchKey("wl{}_expire", int_from_str),  # Expire time in s
+        SearchKey("wl{}_expire_tmp", int_from_str),  # Expire time left in s
+        SearchKey("wl{}_gn_wbl_enable"),
+        SearchKey("wl{}_gn_wbl_rule"),
+        SearchKey("wl{}_gn_wbl_type"),
+        SearchKey("wl{}_hwaddr"),  # MAC address
+        SearchKey("wl{}_ifname"),  # Interface name
+        SearchKey("wl{}_infra"),
+        SearchKey("wl{}_key"),
+        SearchKey("wl{}_key1"),
+        SearchKey("wl{}_key2"),
+        SearchKey("wl{}_key3"),
+        SearchKey("wl{}_key4"),
+        SearchKey("wl{}_lanaccess", bool_from_any),  # LAN access
+        SearchKey("wl{}_maclist"),
+        SearchKey("wl{}_macmode"),
+        SearchKey("wl{}_maxassoc", int_from_str),
+        SearchKey("wl{}_mbss"),
+        SearchKey("wl{}_mfp"),
+        SearchKey("wl{}_mode"),
+        SearchKey("wl{}_net_reauth", int_from_str),
+        SearchKey("wl{}_preauth"),
+        SearchKey("wl{}_radio", bool_from_any),
+        SearchKey("wl{}_radius_ipaddr"),
+        SearchKey("wl{}_radius_key"),
+        SearchKey("wl{}_radius_port", int_from_str),
+        SearchKey("wl{}_sae_anti_clog_threshold"),
+        SearchKey("wl{}_sae_groups"),
+        SearchKey("wl{}_sae_sync"),
+        SearchKey("wl{}_ssid"),  # SSID
+        SearchKey("wl{}_sta_retry_time"),
+        SearchKey("wl{}_sync_node", bool_from_any),  # Sync AiMesh nodes
+        SearchKey("wl{}_unit"),  # GWLAN unit id
+        SearchKey("wl{}_wep", bool_from_any),
+        SearchKey("wl{}_wep_x", bool_from_any),
+        SearchKey("wl{}_wfi_enable", bool_from_any),
+        SearchKey("wl{}_wfi_pinmode"),
+        SearchKey("wl{}_wme"),
+        SearchKey("wl{}_wme_bss_disable", bool_from_any),
+        SearchKey("wl{}_wpa_gtk_rekey"),
+        SearchKey("wl{}_wpa_psk"),  # Password
+        SearchKey("wl{}_wps_mode"),
+    ],
+}
+MAP_OVPN_STATUS = {
+    -1: "error",
+    0: "disconnected",
+    1: "connecting",
+    2: "connected",
+}
+MAP_PORTS: tuple[Key, ...] = ()
+RANGE_CPU_CORES = range(1, 9)  # 8 cores from 1 to 8
+RANGE_GWLAN = range(1, 4)  # 3 guest WLANs from 1 to 3 per each WLAN
+RANGE_OVPN_CLIENTS = range(1, 6)  # 5 Open VPN clients from 1 to 5
+RANGE_WLAN = range(0, 4)  # 4 WLANs from 0 to 3
+
+### TYPES
+
+PORT_TYPES = {
+    "L": LAN,
+    "U": USB,
+    "W": WAN,
+}
+
+SPEED_TYPES = {
+    "X": 0,
+    "M": 100,
+    "G": 1000,
+    "Q": 2500,
+}
+
+TRAFFIC_TYPE: tuple[str, ...] = (
+    RX,
+    TX,
+)
+
+WLAN_TYPE: dict[str, str] = {
+    WLAN_2GHZ: 0,
+    WLAN_5GHZ: 1,
+    WLAN_5GHZ2: 2,
+    WLAN_6GHZ: 3,
+}
+
+### CONVERTERS
+
+CONVERTERS = {
+    PORT_STATUS: [
+        Key("is_on", STATE, method=bool_from_any),
+        Key("max_rate", method=int_from_str),
+        Key(LINK_RATE, method=int_from_str),
+    ]
+}
 
 ### CONSTANTS, DATA TYPES AND COMMON PARAMETERS
 CONST_BITSINBYTE = 8
 CONST_PERCENTS = 100
 CONST_ZERO = 0.0
 
+
+# CHECK LEGACY FROM HERE -->
+
 DATA_FREE = "free"
 DATA_TOTAL = "total"
 DATA_USAGE = "usage"
@@ -78,7 +402,7 @@ KEY_VPN = "VPN"
 KEY_WAN = "WAN"
 
 # OUR REGEX
-REGEX_VARIABLES = '([a-zA-Z0-9\_-]+)\s*=\s*"(.*?)"(?=;)'
+REGEX_VARIABLES = r'([a-zA-Z0-9\_-]+)\s*=\s*"(.*?)"(?=;)'
 
 # VALUES TO IGNORE
 VALUES_TO_IGNORE = [str(), "None", "0.0.0.0"]
@@ -116,6 +440,8 @@ AR_DEVICE_ATTRIBUTES_LIST: tuple[Key, ...] = (
 AR_DEVICE_IDENTITY: tuple[Key, ...] = (
     Key("serial_no", "serial"),
     Key("label_mac", "mac"),
+    Key("lan_hwaddr", "lan_mac"),
+    Key("wan_hwaddr", "wan_mac"),
     Key("productid", "model"),
     Key("firmver", "fw_major"),
     Key("buildno", "fw_minor"),
@@ -303,21 +629,14 @@ AR_KEY_PARENTAL_CONTROL = Key(
 AR_PATH = {
     "apply": "apply.cgi",
     "command": "applyapp.cgi",
-    "devicemap": "ajax_status.xml",
     "devices_update": "update_clients.asp",
-    "firmware": "detect_firmware.asp",
     "get": "appGet.cgi",
     "ledg": "set_ledg.cgi",
     "login": "login.cgi",
     "logout": "Logout.asp",
     "networkmap": "update_networkmapd.asp",
-    "onboarding": "ajax_onboarding.asp",
-    "ports": "ajax_ethernet_ports.asp",
     "rgb": "light_effect.html",
     "state": "state.js",
-    "sysinfo": "ajax_sysinfo.asp",
-    "temperature": "ajax_coretmp.asp",
-    "vpn": "ajax_vpn_status.asp",
 }
 
 AR_VPN_STATUS = {
@@ -329,7 +648,7 @@ AR_VPN_STATUS = {
 
 ### ASUSWRT SERVICES
 AR_HOOK_TEMPLATE = "{}({});"
-AR_HOOK_DEVICES = "{}()".format(AR_KEY_DEVICES)
+AR_HOOK_DEVICES = f"{AR_KEY_DEVICES}()"
 
 AR_FIRMWARE_CHECK_COMMAND = "firmware_check"
 AR_SERVICE_COMMAND: dict[str, str] = {
@@ -705,13 +1024,6 @@ NVRAM_LIST = {
     ],
 }
 
-MONITOR_MAIN = {
-    "cpu_usage": "appobj",
-    "memory_usage": "appobj",
-    "netdev": "appobj",
-    "wanlink_state": "appobj",
-}
-
 PORT_TYPE = [
     "LAN",
     "WAN",
@@ -727,28 +1039,28 @@ DEFAULT_ACTION_MODE = "apply"
 
 # These values are just stored directly in this order in the corresponding node
 DEVICEMAP_BY_INDEX = {
-    "WAN": {
+    WAN: {
         "wan": [
             "status",
             "sbstatus",
             "auxstatus",
         ],
     },
-    "WAN0": {
+    f"{WAN}0": {
         "first_wan": [
             "status",
             "sbstatus",
             "auxstatus",
         ],
     },
-    "WAN1": {
+    f"{WAN}1": {
         "second_wan": [
             "status",
             "sbstatus",
             "auxstatus",
         ],
     },
-    "USB": {
+    USB: {
         "usb": [
             "status",
         ],
@@ -759,7 +1071,7 @@ DEVICEMAP_BY_INDEX = {
 # Not implemented yet
 # psta:wlc_state=0;wlc_state_auth=0;
 DEVICEMAP_SPECIAL = {
-    "WAN": {
+    WAN: {
         "psta": [
             "wlc_state",
             "wlc_state_auth",
@@ -769,7 +1081,7 @@ DEVICEMAP_SPECIAL = {
 
 # These values are stored as "key=value"
 DEVICEMAP_GENERAL = {
-    "WAN": {
+    WAN: {
         "wan": [
             "monoClient",
             "wlc_state",
@@ -796,7 +1108,7 @@ DEVICEMAP_GENERAL = {
             "le_restart_httpd",
         ],
     },
-    "VPN": {
+    VPN: {
         "vpn": [
             "vpnc_proto",
             "vpnc_state_t",
@@ -814,22 +1126,22 @@ DEVICEMAP_GENERAL = {
             "vpn_client5_errno",
         ],
     },
-    "SYS": {
+    SYS: {
         "sys": [
             "uptimeStr",
         ],
     },
-    "QTN": {
+    QTN: {
         "qtn": [
             "qtn_state",
         ],
     },
-    "USB": {
+    USB: {
         "usb": [
             "modem_enable",
         ],
     },
-    "WAN0": {
+    f"{WAN}0": {
         "wan": [
             "wan0_enable",
             "wan0_realip_state",
@@ -837,7 +1149,7 @@ DEVICEMAP_GENERAL = {
             "wan0_realip_ip",
         ],
     },
-    "WAN1": {
+    f"{WAN}1": {
         "wan": [
             "wan1_enable",
             "wan1_realip_state",
@@ -845,7 +1157,7 @@ DEVICEMAP_GENERAL = {
             "wan1_realip_ip",
         ],
     },
-    "SIM": {
+    SIM: {
         "sim": [
             "sim_state",
             "sim_signal",
@@ -862,12 +1174,12 @@ DEVICEMAP_GENERAL = {
             "modem_sim_order",
         ],
     },
-    "DHCP": {
+    DHCP: {
         "dhcp": [
             "dnsqmode",
         ],
     },
-    "DIAG": {
+    DIAG: {
         "diag": [
             "diag_dblog_enable",
             "diag_dblog_remaining",
@@ -877,7 +1189,7 @@ DEVICEMAP_GENERAL = {
 
 # This should be cleared from the data
 DEVICEMAP_CLEAR = {
-    "WAN": {
+    WAN: {
         "data_rate_info_2g": '"',
         "data_rate_info_5g": '"',
         "data_rate_info_5g_2": '"',
