@@ -4,9 +4,10 @@ from __future__ import annotations
 
 import re
 from datetime import datetime, timedelta, timezone
-from dateutil.parser import parse as dtparse
 from typing import Any
 
+from dateutil.parser import parse as dtparse
+
 from asusrouter.error import AsusRouterValueError
 
 BOOL_FALSE: tuple[str, ...] = (
@@ -30,15 +31,19 @@ REGEX_MAC = re.compile("^([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})")
 CONNECTION_TYPE = {
     "2G": 1,
     "5G": 2,
+    "5G1": 3,
     "5G2": 3,
     "6G": 4,
 }
 
 
-def int_from_str(raw: str, base: int = 10) -> int:
-    """Convert string to integer"""
+def int_from_str(raw: str | int, base: int = 10) -> int:
+    """Convert to integer"""
 
-    if type(raw) != str:
+    if isinstance(raw, int):
+        return raw
+
+    if not isinstance(raw, str):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
 
     raw = raw.strip()
@@ -49,22 +54,25 @@ def int_from_str(raw: str, base: int = 10) -> int:
     try:
         return int(raw, base=base)
     except ValueError as ex:
-        raise AsusRouterValueError(ERROR_VALUE.format(raw, str(ex)))
+        raise AsusRouterValueError(ERROR_VALUE.format(raw, str(ex))) from ex
 
 
 def clean_html(raw: str) -> str:
     """Clean html tags"""
 
-    if type(raw) != str:
+    if not isinstance(raw, str):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
 
-    return re.sub("<([\/ibu]+)>", "", raw)
+    return re.sub(r"<([\/ibu]+)>", "", raw)
 
 
-def bool_or_int(raw: str, base: int = 10) -> bool | int:
+def bool_or_int(raw: str | bool, base: int = 10) -> bool | int:
     """Convert to bool or to int"""
 
-    if type(raw) != str:
+    if isinstance(raw, bool):
+        return raw
+
+    if not isinstance(raw, str):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
 
     raw = clean_html(raw)
@@ -84,7 +92,7 @@ def bool_or_int(raw: str, base: int = 10) -> bool | int:
 def float_from_str(raw: str) -> float:
     """Convert striing to float"""
 
-    if type(raw) != str:
+    if not isinstance(raw, str):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
 
     raw = raw.strip()
@@ -98,7 +106,7 @@ def float_from_str(raw: str) -> float:
     try:
         value = float(raw)
     except ValueError as ex:
-        raise AsusRouterValueError(ERROR_VALUE.format(raw, str(ex)))
+        raise AsusRouterValueError(ERROR_VALUE.format(raw, str(ex))) from ex
 
     return value
 
@@ -134,19 +142,28 @@ def bool_from_any(raw: str | int | float | bool) -> bool | None:
 def int_from_bool(raw: bool) -> int:
     """Convert boolean value to 0 or 1"""
 
-    if type(raw) != bool:
+    if not isinstance(raw, bool):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
 
     return 1 if raw else 0
 
 
+def none_or_any(raw: Any) -> None | Any:
+    """Return value or process it with none_or_str"""
+
+    if isinstance(raw, str):
+        return none_or_str(raw)
+
+    return raw
+
+
 def none_or_str(raw: str) -> str | None:
     """Returns either string or None if string is empty"""
 
     if raw is None:
         return None
 
-    if type(raw) != str:
+    if not isinstance(raw, str):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
 
     if raw.strip() == str():
@@ -158,7 +175,7 @@ def none_or_str(raw: str) -> str | None:
 def exists_or_not(raw: str) -> bool:
     """Returns whether value exists or is empty"""
 
-    if type(raw) != str:
+    if not isinstance(raw, str):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
 
     if raw.strip() == str():
@@ -168,9 +185,10 @@ def exists_or_not(raw: str) -> bool:
 
 
 def timedelta_long(raw: str) -> timedelta:
-    """Transform connection timedelta of the device to a proper datetime object when the device was connected"""
+    """Transform connection timedelta of the device to a proper
+    datetime object when the device was connected"""
 
-    if type(raw) != str:
+    if not isinstance(raw, str):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
 
     raw = raw.strip()
@@ -184,7 +202,7 @@ def timedelta_long(raw: str) -> timedelta:
             hours=int(part[-3]), minutes=int(part[-2]), seconds=int(part[-1])
         )
     except ValueError as ex:
-        raise (AsusRouterValueError(ERROR_VALUE.format(raw, str(ex))))
+        raise (AsusRouterValueError(ERROR_VALUE.format(raw, str(ex)))) from ex
 
 
 def time_from_delta(raw: str) -> datetime:
@@ -198,7 +216,7 @@ def time_from_delta(raw: str) -> datetime:
 def datetime_from_str(raw: str) -> datetime:
     """Transform string to datetime"""
 
-    if type(raw) != str:
+    if not isinstance(raw, str):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
     raw = raw.strip()
     if raw == str():
@@ -213,7 +231,7 @@ def datetime_from_str(raw: str) -> datetime:
 def is_mac_address(raw: str) -> bool:
     """Checks if string is MAC address"""
 
-    if type(raw) != str:
+    if not isinstance(raw, str):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
 
     raw = raw.strip()
@@ -230,10 +248,10 @@ def is_mac_address(raw: str) -> bool:
 def service_support(raw: str) -> list[str]:
     """Get the list of the supported services"""
 
-    if type(raw) != str:
+    if not isinstance(raw, str):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
 
-    services = list()
+    services = []
 
     if raw == str():
         return services
@@ -247,7 +265,7 @@ def service_support(raw: str) -> list[str]:
 def ovpn_remote_fom_str(raw: str) -> dict[str, Any]:
     """Get OpenVPN remote data"""
 
-    if type(raw) != str:
+    if not isinstance(raw, str):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
 
     try:
@@ -269,7 +287,7 @@ def ovpn_remote_fom_str(raw: str) -> dict[str, Any]:
 def to_snake_case(raw: str):
     """Convert string to snake_case"""
 
-    if type(raw) != str:
+    if not isinstance(raw, str):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
 
     string = (
@@ -285,7 +303,7 @@ def to_snake_case(raw: str):
 def as_list(raw: Any):
     """Convert object into list of objects"""
 
-    if type(raw) != list:
+    if not isinstance(raw, list):
         return [raw]
 
     return raw
@@ -294,7 +312,7 @@ def as_list(raw: Any):
 def onboarding_connection(raw: str) -> dict[str, int]:
     """Convert connection type from onboarding type to native type"""
 
-    if type(raw) != str:
+    if not isinstance(raw, str):
         raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
 
     if raw == "wired_mac":
@@ -306,7 +324,7 @@ def onboarding_connection(raw: str) -> dict[str, int]:
     try:
         temp = raw.split("_")
         return {
-            "connection_type": CONNECTION_TYPE[temp[0]],
+            "connection_type": CONNECTION_TYPE.get(temp[0]),
             "guest": temp[1] if len(temp) > 1 else 0,
         }
     except Exception as ex:
